function [ids_core_sources,ids_core_sources_description,varargout] = ...
  tcv_get_ids_core_sources(shot,ids_equil_empty,gdat_params,varargin)
%
% [ids_core_sources,ids_core_sources_description,varargout] = ...
%     tcv_get_ids_core_sources(shot,ids_equil_empty,gdat_params,varargin);
%
%
% gdat_params: gdat_data.gdat_params to get all params passed from original call
%

machine = 'tcv';
tens_time = -1;
tens_rho = -0.1;

if exist('gdat_params','var')
  [ids_core_sources, params_core_sources] = ...
    tcv_ids_headpart(shot,ids_equil_empty,'cores_sources','gdat_params',gdat_params,varargin{:});
else
  [ids_core_sources, params_core_sources] = ...
    tcv_ids_headpart(shot,ids_equil_empty,'cores_sources',varargin{:});
  aa=gdat_tcv;
  gdat_params = aa.gdat_params; % to get default params
end
params_eff_ref = gdat_params; params_eff_ref.doplot=0;
try params_eff_ref=rmfield(params_eff_ref,'source');catch;end % make sure no source (from ids def)

% initialize description
ids_core_sources_description = [];

%%
% save source template to fill for differnet source options
source_template = ids_core_sources.source{1};
profiles_template = source_template.profiles_1d{1};
globals_template = source_template.global_quantities{1};
%% get time grid and data from gdat 
ohm_gdat = gdat(shot,'ohm_data');
powers_gdat = gdat(shot,'powers');
% fix time grid to ohm time grid, same as bs

last_index = 0;
%% initialize source from template
% ohm
ohm_t_grid = powers_gdat.ohm.t; ohm_n_t = numel(ohm_t_grid);

id_ohm.description = 'Source from ohmic heating';
id_ohm.index = 7; id_ohm.name = 'ohmic';
ids_core_sources.source{last_index+1} = source_template;
ids_core_sources.source{last_index+1}.identifier = id_ohm;
ids_core_sources.source{last_index+1}.profiles_1d(1:ohm_n_t) = {profiles_template};
ids_core_sources.source{last_index+1}.global_quantities(1:ohm_n_t) = {globals_template};
ohm_data = ohm_gdat.ohm.ohm_data;

%fill profiles for times n t_grid

for ii = 1:ohm_n_t
  % ohm
  % 1d_profiles
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.time = ohm_t_grid(ii);
  ids_core_sources.source{last_index+1}.profiles_1d{ii}.j_parallel = ...
    ohm_data.cd_dens.data(:,ii)';
  % globals
  ids_core_sources.source{last_index+1}.global_quantities{ii}.time = ohm_t_grid(ii);
  ids_core_sources.source{last_index+1}.global_quantities{ii}.power = powers_gdat.ohm.data(ii);
end

last_index = last_index+1;  % add if statement to only increment if ohmic source has been added

%% bs
bs_gdat = gdat(shot,'bs_data'); bs_data = bs_gdat.bs.bs_data;
bs_t_grid = bs_gdat.bs.t; bs_n_t = numel(bs_t_grid);

id_bs.description = 'Bootstrap current';
id_bs.index = 13; id_bs.name = 'bootstrap_current';
ids_core_sources.source{last_index+1} = source_template;
ids_core_sources.source{last_index+1}.identifier = id_bs;
ids_core_sources.source{last_index+1}.profiles_1d(1:ohm_n_t) = {profiles_template};
ids_core_sources.source{last_index+1}.global_quantities(1:ohm_n_t) = {globals_template};


% fill profiles for times n t_grid
for ii = 1:bs_n_t
   ids_core_sources.source{last_index+1}.profiles_1d{ii}.time = bs_t_grid(ii);
   ids_core_sources.source{last_index+1}.profiles_1d{ii}.j_parallel = ...
     bs_data.cd_dens.data(:,ii)'; 
end
 
last_index = last_index+1;  % add if statement to only increment if bs source has been added

%% ec
% load data
ec_gdat = gdat(shot,'ec_data'); ec_data = ec_gdat.ec.ec_data;
ec_time = ec_data.p_dens.t;
n_ec_time = numel(ec_time);
ec_t_grid = powers_gdat.ec.t; ec_n_t = numel(ec_t_grid);

nb_launchers = size(ec_gdat.ec.ec_data.rho_max_pow_dens.data,1);
id_ec.description = 'Sources from electron cyclotron heating and current drive';
id_ec.index = 3; id_ec.name = 'ec';
for i_lau = 1:nb_launchers
    ids_core_sources.source{last_index+i_lau} = source_template;
    ids_core_sources.source{last_index+i_lau}.identifier = id_ec;
    ids_core_sources.source{last_index+i_lau}.profiles_1d(1:ec_n_t) = {profiles_template};
    ids_core_sources.source{last_index+i_lau}.global_quantities(1:ec_n_t) = {globals_template};
end
 
ec_total_pow = transpose(powers_gdat.ec.data(:,nb_launchers+1)); %use power from powers_gdat(injected ec power) instead of ec_data power
ec_total_pow(isnan(ec_total_pow)) = 0;
ec_total_cur = ec_data.cd_tot.data(nb_launchers+1,:);
ec_total_cur(isnan(ec_total_cur)) = 0;

for i_lau = 1:nb_launchers
    for ii = 1:ec_n_t
        ids_core_sources.source{last_index+i_lau}.profiles_1d{ii}.time = ec_t_grid(ii); % profiles time 
        ids_core_sources.source{last_index+i_lau}.global_quantities{ii}.time = ec_t_grid(ii); % globals time
    
        % globals
        ids_core_sources.source{last_index+i_lau}.global_quantities{ii}.time = ec_t_grid(ii);
        ids_core_sources.source{last_index+i_lau}.global_quantities{ii}.power = ec_total_pow(ii);
        ids_core_sources.source{last_index+i_lau}.global_quantities{ii}.current_parallel = ec_total_cur(ii);
    end
end

% interpoating p_dens profiles from 'ec_time' grid (toray time nodes) to 'ec_t_grid' (injected power time)

p_dens = ec_data.p_dens.data;
p_integrated = ec_data.p_integrated.data;
cd_dens = ec_data.cd_dens.data;
cd_integrated = ec_data.cd_integrated.data;

p_ec_injected = powers_gdat.ec.data;
it = iround_os(ec_t_grid,ec_time);
sparse_p_ec_injected = p_ec_injected(it,:); % injected ec power vals corresponding to ec_time grid
rho_grid = size(p_dens, 1);
% calculate normalised profiles on ec_time grid
norm_p_dens = zeros(rho_grid, nb_launchers+1, n_ec_time);
norm_p_integrated = zeros(rho_grid, nb_launchers+1, n_ec_time);
norm_cd_dens = zeros(rho_grid, nb_launchers+1, n_ec_time);
norm_cd_integrated = zeros(rho_grid, nb_launchers+1, n_ec_time);
for t = 1:n_ec_time
    norm_p_dens_temp = zeros(rho_grid, nb_launchers+1);
    norm_p_integrated_temp = zeros(rho_grid, nb_launchers+1);
    norm_cd_dens_temp = zeros(rho_grid, nb_launchers+1);
    norm_cd_integrated_temp = zeros(rho_grid, nb_launchers+1);
    for launcher = 1:nb_launchers+1
        norm_p_dens_temp(:, launcher) = p_dens(:, launcher, t) ./ sparse_p_ec_injected(t, launcher);
        norm_p_integrated_temp(:, launcher) = p_integrated(:, launcher, t) ./ sparse_p_ec_injected(t, launcher);
        norm_cd_dens_temp(:, launcher) = cd_dens(:, launcher, t) ./ sparse_p_ec_injected(t, launcher);
        norm_cd_integrated_temp(:, launcher) = cd_integrated(:, launcher, t) ./ sparse_p_ec_injected(t, launcher);
    end
    norm_p_dens(:,:,t) = norm_p_dens_temp;
    norm_p_integrated(:,:,t) = norm_p_integrated_temp;
    norm_cd_dens(:,:,t) = norm_cd_dens_temp;
    norm_cd_integrated(:,:,t) = norm_cd_integrated_temp;
end

% interpolate normalised p_dens profiles on ec_t_grid 
interp_norm_p_dens = zeros(rho_grid, nb_launchers+1, ec_n_t);
interp_norm_p_integrated = zeros(rho_grid, nb_launchers+1, ec_n_t);
interp_norm_cd_dens = zeros(rho_grid, nb_launchers+1, ec_n_t);
interp_norm_cd_integrated = zeros(rho_grid, nb_launchers+1, ec_n_t);
for rho = 1:rho_grid
    for launcher = 1:nb_launchers+1
        profile_p_dens = squeeze(norm_p_dens(rho, launcher, :));
        profile_p_integrated = squeeze(norm_p_integrated(rho, launcher, :));
        profile_cd_dens = squeeze(norm_cd_dens(rho, launcher, :));
        profile_cd_integrated = squeeze(norm_cd_integrated(rho, launcher, :));
        interp_profile_p_dens = interp1(ec_time, profile_p_dens, ec_t_grid);
        interp_profile_p_integrated = interp1(ec_time, profile_p_integrated, ec_t_grid);
        interp_profile_cd_dens = interp1(ec_time, profile_cd_dens, ec_t_grid);
        interp_profile_cd_integrated = interp1(ec_time, profile_cd_integrated, ec_t_grid);
        interp_norm_p_dens(rho, launcher, :) = interp_profile_p_dens;
        interp_norm_p_integrated(rho, launcher, :) = interp_profile_p_integrated;
        interp_norm_cd_dens(rho, launcher, :) = interp_profile_cd_dens;
        interp_norm_cd_integrated(rho, launcher, :) = interp_profile_cd_integrated;
    end
end

% normalised & interpolated profiles * p_ec_injected on ec_t_grid
interp_p_dens = zeros(rho_grid, nb_launchers+1, ec_n_t);
interp_p_integrated = zeros(rho_grid, nb_launchers+1, ec_n_t);
interp_cd_dens = zeros(rho_grid, nb_launchers+1, ec_n_t);
interp_cd_integrated = zeros(rho_grid, nb_launchers+1, ec_n_t);
for t = 1:numel(ec_t_grid)
    unnormalised_p_dens = zeros(rho_grid, nb_launchers+1);
    unnormalised_p_integrated = zeros(rho_grid, nb_launchers+1);
    unnormalised_cd_dens = zeros(rho_grid, nb_launchers+1);
    unnormalised_cd_integrated = zeros(rho_grid, nb_launchers+1);
    for launcher = 1:nb_launchers+1
        unnormalised_p_dens(:,launcher) = interp_norm_p_dens(:,launcher,t).* p_ec_injected(t, launcher);
        unnormalised_p_integrated(:,launcher) = interp_norm_p_integrated(:,launcher,t).* p_ec_injected(t, launcher);
        unnormalised_cd_dens(:,launcher) = interp_norm_cd_dens(:,launcher,t).* p_ec_injected(t, launcher);
        unnormalised_cd_integrated(:,launcher) = interp_norm_cd_integrated(:,launcher,t).* p_ec_injected(t, launcher);
    end
    interp_p_dens(:,:,t) = unnormalised_p_dens;
    interp_p_integrated(:,:,t) = unnormalised_p_integrated;
    interp_cd_dens(:,:,t) = unnormalised_cd_dens;
    interp_cd_integrated(:,:,t) = unnormalised_cd_integrated;
end

for ii = 1:ec_n_t
    for i = 1:nb_launchers
        ids_core_sources.source{last_index+i}.profiles_1d{ii}.grid.rho_tor_norm = ...
          ec_data.p_dens.rhotor_norm(1,:);
        ids_core_sources.source{last_index+i}.profiles_1d{ii}.electrons.energy = ...
          interp_p_dens(:,i,ii);
        ids_core_sources.source{last_index+i}.profiles_1d{ii}.electrons.power_inside = ...
          interp_p_integrated(:,i,ii);
      % current density (to adapt to <J.B>/B0)
        ids_core_sources.source{last_index+i}.profiles_1d{ii}.j_parallel = ...
          interp_cd_dens(:,i,ii);
        % integrated current density (to adapt to INTEGRAL(<J.B>/B0)*ds_phi)
        ids_core_sources.source{last_index+i}.profiles_1d{ii}.current_parallel_inside = ...
          interp_cd_integrated(:,i,ii); 
      
    end
end
last_index = last_index+nb_launchers;

%% nbi  
id_nbi.description = 'Source from Neutral Beam Injection';
id_nbi.index = 2; id_nbi.name = 'nbi';
%ids_core_sources.source{} = source_template;
%ids_core_sources.source{}.identifier = id_nbi;
%ids_core_sources.source{}.profiles_1d(1:n_t) = {profiles_template};
%ids_core_sources.source{}.global_quantities(1:n_t) = {globals_template};


% total  
id_total.description = 'Total source; combines all sources';
id_total.index = 1; id_total.name = 'total';
%ids_core_sources.source{} = source_template;
%ids_core_sources.source{}.identifier = id_total;
%ids_core_sources.source{}.profiles_1d(1:n_t) = {profiles_template};
%ids_core_sources.source{}.global_quantities(1:n_t) = {globals_template};


%% add descriptions for profiles_1d

desc.source = 'available by now: ohm [id=7], bootstrap [id=13], ec [id=3] (if present in shot)';
desc.profiles_1d.j_parallel = 'parallel current density';
desc.profiles_1d.current_parallel_inside = 'integrated parallel current density';
desc.globals.power = 'total power coupled to the plasma';
desc.globals.current_parallel = 'total parallel current driven';
ids_core_sources_description = desc;

%%
if nargin <= 2
  ids_core_sources.code.name = ['tcv_get_ids_core_sources, within gdat, with shot= ' num2str(params_core_sources.shot) ];
else
  ids_core_sources.code.name = ['tcv_get_ids_core_sources, within gdat, with shot= ' num2str(params_core_sources.shot) '; varargin: ' varargin{:}];
end
ids_core_sources_description.code.name = ids_core_sources.code.name;

ids_core_sources.code.output_flag = zeros(size(ids_core_sources.time));

% cocos automatic transform
if ~isempty(which('ids_generic_cocos_nodes_transformation_symbolic'))
  [ids_core_sources,cocoscoeff]=ids_generic_cocos_nodes_transformation_symbolic(ids_core_sources,'core_sources',gdat_params.cocos_in, ...
          gdat_params.cocos_out,gdat_params.ipsign_out,gdat_params.b0sign_out,gdat_params.ipsign_in,gdat_params.b0sign_in, ...
          gdat_params.error_bar,gdat_params.nverbose);

end